﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text.RegularExpressions;

namespace Smdn.Formats.Ini {
  internal static class Parser {
    public static Document Parse(TextReader reader)
    {
      var document = new Document();
      var section = string.Empty; // default section name

      for (;;) {
        var line = reader.ReadLine();

        if (line == null)
          break;

        // 空行は読み飛ばす
        if (string.Empty.Equals(line))
          continue;

        // コメント行は読み飛ばす
        if (line.StartsWith(";"))
          continue;

        if (line.StartsWith("#"))
          continue;

        var matchEntry = regexEntry.Match(line);

        if (matchEntry.Success) {
          // name=valueの行
          document[section][matchEntry.Groups["name"].Value.Trim()] = matchEntry.Groups["value"].Value.Trim();
          continue;
        }

        var matchSection = regexSection.Match(line);

        if (matchSection.Success) {
          // [section]の行
          section = matchSection.Groups["section"].Value;
          continue;
        }
      }
      return document;
    }

    private static readonly Regex regexSection = new Regex(@"^[\s\t]*\[(?<section>[^\]]+)\].*$", RegexOptions.Singleline);
    private static readonly Regex regexEntry = new Regex(@"^[\s\t]*(?<name>[^=]+)=(?<value>.*?)([\s\t]+;(?<comment>.*))?$", RegexOptions.Singleline);
  }
}
