using System;
using System.Collections.Generic;

using Smdn.Protocols.Imap4.Protocol;

namespace Smdn.Protocols.Imap4 {
  // DataItem types:
  //   ImapDataItem
  //     => handles 'data item names'
  //     ImapFetchDataItem
  //       => handles 'message data item names or macro'
  //     ImapSearchCriteria
  //       => handles 'searching criteria'
  //     ImapStatusDataItem
  //       => handles 'status data item names'
  //     ImapStoreDataItem
  //       => handles 'message data item name' and 'value for message data item'

  public abstract class ImapDataItem {
    protected ImapString[] DataItems {
      get { return dataItems; }
    }

    protected ImapDataItem(params ImapString[] dataItems)
    {
      this.dataItems = dataItems;
    }

    protected static ImapString[] GetCombinedDataItems(ImapDataItem x, ImapDataItem y)
    {
      return GetCombinedDataItems(x.dataItems, y.dataItems);
    }

    protected static ImapString[] GetCombinedDataItems(ImapString[] x, ImapString[] y)
    {
      var combinedDataItems = new List<ImapString>(x);

      combinedDataItems.AddRange(y);

      return combinedDataItems.ToArray();
    }

    internal virtual ImapStringList ToStringList()
    {
      return new ImapStringList(dataItems);
    }

    public override string ToString()
    {
      return ToStringList().ToString();
    }

    private readonly ImapString[] dataItems;
  }
}