using System;
using System.Collections.Generic;
using System.Text;

namespace Smdn.Protocols.Imap4 {
  public class ImapMessage {
    public ImapMailbox Mailbox {
      get { return mailbox; }
      internal set { mailbox = value; }
    }

    public int Sequence {
      get { return sequence; }
    }

    public ImapUri Uri {
      get
      {
        if (uid == 0)
          return null;
        else
          return new ImapUri(string.Format("{0}/;UID={1}", mailbox.Uri, uid));
      }
    }

    public int Uid {
      get { return uid; }
      internal set { uid = value; }
    }

    public ImapEnvelope Envelope {
      get { return envelope; }
      internal set { envelope = value; }
    }

    public ImapBodyStructure BodyStructure {
      get { return bodyStructure; }
      internal set { bodyStructure = value; }
    }

    public int BodySize {
      get { return bodySize; }
      internal set { bodySize = value; }
    }

    public string Body {
      get { return body; }
      internal set { body = value; }
    }

    public ImapMessageFlag Flags {
      get { return flags; }
      internal set { flags = value; }
    }

    public DateTime InternalDate {
      get { return internalDate; }
      internal set { internalDate = value; }
    }

    internal ImapMessage(int sequence)
    {
      if (sequence <= 0)
        throw new ArgumentOutOfRangeException("sequence must be non-zero positive number");

      this.sequence = sequence;
    }

    public ImapMessage(string body)
      : this(body, DateTime.MinValue, null)
    {
    }

    public ImapMessage(string body, DateTime internalDate)
      : this(body, internalDate, null)
    {
    }

    public ImapMessage(string body, ImapMessageFlag flags)
      : this(body, DateTime.MinValue, flags)
    {
    }

    public ImapMessage(string body, DateTime internalDate, ImapMessageFlag flags)
    {
      if (body == null)
        throw new ArgumentNullException("body");

      this.sequence = 0; // invalid sequence number
      this.body = body;
      this.internalDate = internalDate;
      this.flags = flags;
    }

    public override string ToString()
    {
      return string.Format("{{Uid={0}, Sequence={1}, Envelope={2}, BodyStructure={3}, BodySize={4}, Body={5}, Flags={6}, InternalDate={7}}}",
                           uid,
                           sequence,
                           envelope,
                           bodyStructure,
                           bodySize,
                           body,
                           flags,
                           internalDate);
    }

    private ImapMailbox mailbox;
    private readonly int sequence;
    private int uid = 0;
    private ImapEnvelope envelope = null;
    private ImapBodyStructure bodyStructure = null;
    private int bodySize = 0;
    private string body = string.Empty;
    private ImapMessageFlag flags = null;
    private DateTime internalDate = DateTime.MinValue;
  }
}