using System;

namespace Smdn.Protocols.Imap4 {
  // 2.3.2. Flags Message Attribute
  public class ImapMessageFlag {
    public ImapSystemFlags SystemFlags {
      get { return systemFlags; }
      set { systemFlags = value; }
    }

    public string[] Keywords {
      get { return keywords; }
      set
      {
        ImapConverter.ValidateKeywords(keywords);
        keywords = value;
      }
    }

    public ImapMessageFlag()
      : this(ImapSystemFlags.None)
    {
    }

    public ImapMessageFlag(ImapSystemFlags systemFlags)
      : this(systemFlags, new string[] {})
    {
    }

    public ImapMessageFlag(params string[] keywords)
      : this(ImapSystemFlags.None, keywords)
    {
    }

    public ImapMessageFlag(ImapSystemFlags systemFlags, params string[] keywords)
    {
      ImapConverter.ValidateKeywords(keywords);

      this.systemFlags = systemFlags;
      this.keywords = keywords;
    }

    public bool Has(ImapSystemFlags systemFlag)
    {
      return (0 != (systemFlags & systemFlag));
    }

    public bool Has(string keyword)
    {
      foreach (var key in keywords) {
        if (keyword.Equals(key, StringComparison.InvariantCultureIgnoreCase))
          return true;
      }

      return false;
    }

    public override string ToString()
    {
      return string.Format("{{SystemFlags={0}, Keywords={1}}}",
                           systemFlags,
                           string.Join(" ", keywords));
    }

    private ImapSystemFlags systemFlags;
    private string[] keywords;
  }
}
