using System;

using Smdn.Protocols.Imap4.Protocol;

namespace Smdn.Protocols.Imap4 {
  // DataItem types:
  //   ImapDataItem
  //     => handles 'data item names'
  //     ImapFetchDataItem
  //       => handles 'message data item names or macro'
  //     ImapSearchCriteria
  //       => handles 'searching criteria'
  //   * ImapStatusDataItem
  //       => handles 'status data item names'
  //     ImapStoreDataItem
  //       => handles 'message data item name' and 'value for message data item'

  // 6.3.10. STATUS Command
  //   status data item names
  public class ImapStatusDataItem : ImapDataItem {
    public static readonly ImapStatusDataItem All
      = new ImapStatusDataItem("MESSAGES", "RECENT", "UIDNEXT", "UIDVALIDITY", "UNSEEN");

    // MESSAGES
    public static readonly ImapStatusDataItem Messages
      = new ImapStatusDataItem("MESSAGES");

    // RECENT
    public static readonly ImapStatusDataItem Recent
      = new ImapStatusDataItem("RECENT");

    // UIDNEXT
    public static readonly ImapStatusDataItem UidNext
      = new ImapStatusDataItem("UIDNEXT");

    // UIDVALIDITY
    public static readonly ImapStatusDataItem UidValidity
      = new ImapStatusDataItem("UIDVALIDITY");

    // UNSEEN
    public static readonly ImapStatusDataItem Unseen
      = new ImapStatusDataItem("UNSEEN");

    public ImapStatusDataItem CombineWith(ImapStatusDataItem dataItem)
    {
      return this + dataItem;
    }

    public static ImapStatusDataItem Combine(ImapStatusDataItem x, ImapStatusDataItem y)
    {
      return x + y;
    }

    public static ImapStatusDataItem operator+ (ImapStatusDataItem x, ImapStatusDataItem y)
    {
      return new ImapStatusDataItem(GetCombinedDataItems(x, y));
    }

    private ImapStatusDataItem(params ImapString[] dataItems)
      : base(dataItems)
    {
    }

    internal override ImapStringList ToStringList()
    {
      return new ImapStringList(new ImapParenthesizedString(base.DataItems));
    }
  }
}