using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;

namespace Smdn.Protocols.Imap4 {
  // StringTypes:
  //   ImapString
  //     => handles 'string'
  //     ImapQuotedString
  //       => handles 'quoted'
  //     ImapLiteralString
  //       => handles 'literal'
  //     ImapStringList
  //       => list of ImapString
  //       ImapParenthesizedString
  //         => handles 'parenthesized list'

  public class ImapStringList : ImapString, IEnumerable<ImapString> {
    public ImapStringList(string val)
      : this(new[] {new ImapString(val)}, false)
    {
    }

    public ImapStringList(params ImapString[] values)
      : this(values, false)
    {
    }

    protected ImapStringList(ImapString[] values, bool parenthesize)
      : base()
    {
      this.values = values;
      this.parenthesize = parenthesize;
    }

    public void Traverse(Action<ImapString> action) {
      foreach (var val in values) {
        if (val is ImapStringList)
          (val as ImapStringList).Traverse(action);
        else
          action(val);
      }
    }

    public IEnumerator<ImapString> GetEnumerator()
    {
      return (values as IEnumerable<ImapString>).GetEnumerator();
    }

    IEnumerator IEnumerable.GetEnumerator()
    {
      return values.GetEnumerator();
    }

    public override string ToString()
    {
      var sb = new StringBuilder();

      if (parenthesize)
        sb.Append("(");

      for (var i = 0; i < values.Length; i++) {
        if (i != 0)
          sb.Append(" "); // SP
        sb.Append(values[i].ToString());
      }

      if (parenthesize)
        sb.Append(")");

      return sb.ToString();
    }

    private readonly ImapString[] values;
    private readonly bool parenthesize;
  }
}
