using System;

namespace Smdn.Protocols.Imap4 {
  public class ImapUriBuilder
  {
    public string Host {
      get; set;
    }

    public int Port {
      // UriBuilder.Port:
      // ポートが URI の一部として指定されない場合、Port プロパティは値 -1 を返します。
      get; // default ImapUri.DefaultPort
      set; // if < -1 throw ArgumentOutOfRangeException
    }

    public string Query {
      get; // starts with '?'
      set;
    }

    public string UserName {
      get;
      set;
    }

    public ImapAuthenticationType Authentication {
      get;
      set;
    }

    public int? UidValidity {
      get;
      set; // must be non-zero positive number
    }

    public int? MessageUid {
      get;
      set; // must be non-zero positive number
    }

    public string Section {
      get;
      set;
    }

    public string Mailbox {
      get;
      set;
    }

    public ImapListType ListType {
      get;
      set;
    }

    public ImapUriBuilder()
      : this(string.Empty, string.Empty, ImapAuthenticationType.None)
    {
    }

    public ImapUriBuilder(ImapUri uri)
    {
    }

    // imap://<iserver>/
    public ImapUriBuilder(string host, string userName, ImapAuthenticationType authentication)
      : this(host, userName, authentication, ImapUri.DefaultPort)
    {
    }

    // imap://<iserver>:port/
    public ImapUriBuilder(string host, string userName, ImapAuthenticationType authentication, int port)
      : this(host, userName, authentication, port, null, null, null)
    {
    }

    // imap://<iserver>/<enc_mailbox>?<enc_search>
    public ImapUriBuilder(string host, string userName, ImapAuthenticationType authentication, string mailbox, string query)
      : this(host, userName, authentication, ImapUri.DefaultPort, mailbox, null, query)
    {
    }

    // imap://<iserver>:port/<enc_mailbox>?<enc_search>
    public ImapUriBuilder(string host, string userName, ImapAuthenticationType authentication, int port, string mailbox, string query)
      : this(host, userName, authentication, port, mailbox, null, query)
    {
    }

    // imap://<iserver>/<enc_mailbox>uidvalidity
    public ImapUriBuilder(string host, string userName, ImapAuthenticationType authentication, string mailbox, int uidValidity)
      : this(host, userName, authentication, ImapUri.DefaultPort, mailbox, uidValidity, string.Empty)
    {
    }

    // imap://<iserver>:port/<enc_mailbox>uidvalidity
    public ImapUriBuilder(string host, string userName, ImapAuthenticationType authentication, int port, string mailbox, int uidValidity)
      : this(host, userName, authentication, port, mailbox, uidValidity, string.Empty)
    {
    }
    // imap://<iserver>/<enc_mailbox>uidvalidity?<enc_search>
    public ImapUriBuilder(string host, string userName, ImapAuthenticationType authentication, string mailbox, int? uidValidity, string query)
      : this(host, userName, authentication, ImapUri.DefaultPort, mailbox, uidValidity, query)
    {
    }

    // imap://<iserver>:port/<enc_mailbox>uidvalidity?<enc_search>
    public ImapUriBuilder(string host, string userName, ImapAuthenticationType authentication, int port, string mailbox, int? uidValidity, string query)
    {
      this.Host = host;
      this.UserName = userName;
      this.Authentication = authentication;
      this.Port = port;
      this.Mailbox = mailbox;
      this.UidValidity = uidValidity;
      this.Query = query;

      throw new NotImplementedException();
    }

    // imap://<iserver>/<enc_list_mailbox>;TYPE=<list_type>
    public ImapUriBuilder(string host, string userName, ImapAuthenticationType authentication, string mailbox, ImapListType listType)
      : this(host, userName, authentication, ImapUri.DefaultPort, mailbox, listType)
    {
    }

    // imap://<iserver>:port/<enc_list_mailbox>;TYPE=<list_type>
    public ImapUriBuilder(string host, string userName, ImapAuthenticationType authentication, int port, string mailbox, ImapListType listType)
    {
      this.Host = host;
      this.UserName = userName;
      this.Authentication = authentication;
      this.Port = port;
      this.Mailbox = mailbox;
      this.ListType = listType;

      throw new NotImplementedException();
    }

    // imap://<iserver>/<enc_mailbox>uidvalidity<iuid>
    public ImapUriBuilder(string host, string userName, ImapAuthenticationType authentication, string mailbox, int? uidValidity, int messageUid)
      : this(host, userName, authentication, ImapUri.DefaultPort, mailbox, uidValidity, messageUid, string.Empty)
    {
    }

    // imap://<iserver>:port/<enc_mailbox>uidvalidity<iuid>
    public ImapUriBuilder(string host, string userName, ImapAuthenticationType authentication, int port, string mailbox, int? uidValidity, int messageUid)
      : this(host, userName, authentication, port, mailbox, uidValidity, messageUid, string.Empty)
    {
    }

    // imap://<iserver>/<enc_mailbox>uidvalidity<iuid>isection
    public ImapUriBuilder(string host, string userName, ImapAuthenticationType authentication, string mailbox, int? uidValidity, int? messageUid, string section)
      : this(host, userName, authentication, ImapUri.DefaultPort, mailbox, uidValidity, messageUid, section)
    {
    }

    // imap://<iserver>:port/<enc_mailbox>uidvalidity<iuid>isection
    public ImapUriBuilder(string host, string userName, ImapAuthenticationType authentication, int port, string mailbox, int? uidValidity, int? messageUid, string section)
    {
      this.Host = host;
      this.UserName = userName;
      this.Authentication = authentication;
      this.Port = port;
      this.Mailbox = mailbox;
      this.UidValidity = uidValidity;
      this.MessageUid = messageUid;
      this.Section = section;

      throw new NotImplementedException();
    }
  }
}
