using System;
using System.Collections.Generic;

using Smdn.Protocols.Imap4.Protocol;

namespace Smdn.Protocols.Imap4.Transaction {
  // Layers:
  //   Client.ImapClient
  //   Client.ImapClientAgent
  //   Session.ImapSessionAgent
  //   Session.ImapSession
  //   Transaction.ImapTransactionAgent
  // * Transaction.ImapTransaction/Transaction.ImapRequest/Transaction.ImapResponse
  //   Protocol.ImapConnection/Protocol.ImapProtocolCommand/Protocol.ImapProtocolResponse
  //   (IMAP4)

  public class ImapRequest {
    public ImapTransactionMethod Method {
      get { return method; }
    }

    public Dictionary<string, ImapString> Arguments {
      get { return arguments; }
    }

    public int Timeout {
      get { return timeout; }
    }

    internal DateTime RequestedTime {
      get; set;
    }

    internal ImapRequest(ImapTransactionMethod method, int timeout)
      : this(method, timeout, null)
    {
    }

    internal ImapRequest(ImapTransactionMethod method, int timeout, IDictionary<string, string> arguments)
    {
      this.method = method;
      this.timeout = timeout;

      if (arguments != null) {
        foreach (var pair in arguments) {
          this.arguments.Add(pair.Key, pair.Value);
        }
      }
    }

    public override string ToString()
    {
      return string.Format("{{Method={0}, Arguments={1}}}", method, arguments.Count);
    }

    private readonly Dictionary<string, ImapString> arguments = new Dictionary<string, ImapString>();
    private readonly ImapTransactionMethod method;
    private readonly int timeout;
  }
}
