using System;
using System.Collections.Generic;

using Smdn.Protocols.Imap4.Protocol;

namespace Smdn.Protocols.Imap4.Transaction.Transactions {
  internal class CapabilityTransaction : TransactionBase {

    // ProcessCapability
    //   ->WaitForCapability
    // WaitForCapability
    //   x Conclude

    public CapabilityTransaction(ImapConnection connection, ImapRequest request)
      : base(connection, request)
    {
    }

    protected override ProcessTransactionDelegate Reset()
    {
      return ProcessCapability;
    }

    private void ProcessCapability()
    {
      Send("capability", WaitForCapability);
    }

    private void WaitForCapability()
    {
      ImapProtocolResponse response;

      if (!Receive(out response))
        return;

      if (response is ImapDataResponse) {
        if (!ImapDataResponseConverter.FromCapabilityResponse(response as ImapDataResponse, out capabilities))
          Conclude(new ImapResponse(ImapTransactionStatus.Forbidden, "capability comprehension error occurred"));
      }
      else if (response is ImapStatusResponse) {
        var status = (response as ImapStatusResponse);

        if (status.Tag != LastTag)
          return;

        if (!capabilities.Has(ImapKnownCapabilities.Imap4Rev1)) {
          Conclude(new ImapResponse(ImapTransactionStatus.BadRequest, "server doesn't support IMAP4rev1"));
        }
        else if (status.Status == ImapProtocolStatus.Ok) {
          Connection.ServerInfo.Capabilities = capabilities;

          Conclude(new ImapCapabilityResponse(ImapTransactionStatus.Ok,
                                              capabilities,
                                              "comprehended server capabilities"));
        }
        else {
          ConcludeError(status);
        }
      }
    }

    private ImapCapability capabilities = null;
  }
}
