using System;

using Smdn.Protocols.Imap4.Protocol;

namespace Smdn.Protocols.Imap4.Transaction.Transactions {
  internal class CreateTransaction : TransactionBase {

    // ProcessCreate
    //   ->WaitForCreate
    // ProcessArgumentNotSetted
    //   x Conclude
    // WaitForCreate
    //   x Conclude

    public CreateTransaction(ImapConnection connection, ImapRequest request)
      : base(connection, request)
    {
    }

    protected override ProcessTransactionDelegate Reset()
    {
      if (!Request.Arguments.ContainsKey("mailbox name"))
        return ProcessArgumentNotSetted;
      else
        return ProcessCreate;
    }

    private void ProcessArgumentNotSetted()
    {
      Conclude(new ImapResponse(ImapTransactionStatus.BadRequest, "arguments 'mailbox name' must be setted"));
    }

    private void ProcessCreate()
    {
      Send("create",
           WaitForCreate,
           new ImapQuotedString(ImapEncoding.ToModifiedUTF7String((string)Request.Arguments["mailbox name"])));
    }

    private void WaitForCreate()
    {
      ImapProtocolResponse response;

      if (!Receive(out response))
        return;

      if (response is ImapStatusResponse) {
        var status = response as ImapStatusResponse;

        if (status.Tag != LastTag)
          return;

        if (status.Status == ImapProtocolStatus.Ok) {
          // get created instance
          var mailbox = ImapMailboxManager.GetMailbox(Connection.ServerInfo, (string)Request.Arguments["mailbox name"]);

          Trace.Dump(mailbox);

          Conclude(new ImapMailboxResponse(ImapTransactionStatus.Ok, mailbox, "mailbox created"));
        }
        else if (status.Status == ImapProtocolStatus.No) {
          // RFC 2193 IMAP4 Mailbox Referrals
          // http://tools.ietf.org/html/rfc2193
          // 4.2. CREATE Referrals
          ImapUri[] referrals;
          if (ImapStatusResponseConverter.FromCreateNoResponse(status, out referrals))
            Conclude(new ImapMailboxReferralResponse(ImapTransactionStatus.MailboxReferral, referrals, status.Text));
          else
            ConcludeError(status);
        }
        else {
          ConcludeError(status);
        }
      }
    }

  }
}
