using System;
using System.Collections.Generic;

using Smdn.Protocols.Imap4.Protocol;

namespace Smdn.Protocols.Imap4.Transaction.Transactions {
  internal class FetchTransaction : TransactionBase {

    // ProcessArgumentNotSetted
    //   x Conclude
    // ProcessFetch
    //   ->WaitForFetch
    // WaitForFetch
    //   x Conclude

    public FetchTransaction(ImapConnection connection, ImapRequest request)
      : base(connection, request)
    {
    }

    protected override ProcessTransactionDelegate Reset()
    {
      if (!Request.Arguments.ContainsKey("message data item names"))
        return ProcessArgumentNotSetted;
      else if (!(Request.Arguments.ContainsKey("uid set") || Request.Arguments.ContainsKey("sequence set")))
        return ProcessArgumentNotSetted;
      else
        return ProcessFetch;
    }

    private void ProcessArgumentNotSetted()
    {
      Conclude(new ImapResponse(ImapTransactionStatus.BadRequest, "arguments 'message data item names' and 'uid set' or 'sequence set' must be setted"));
    }

    private void ProcessFetch()
    {
      if (Request.Arguments.ContainsKey("uid set")) {
      // UID FETCH
        Send("uid fetch",
             WaitForFetch,
             Request.Arguments["uid set"],
             Request.Arguments["message data item names"]);
      }
      else {
        // FETCH
        Send("fetch",
             WaitForFetch,
             Request.Arguments["sequence set"],
             Request.Arguments["message data item names"]);
      }
    }

    private void WaitForFetch()
    {
      ImapProtocolResponse response;

      if (!Receive(out response))
        return;

      if (response is ImapDataResponse) {
        ImapMessage message;
        if (ImapDataResponseConverter.FromFetchCommandResponse(response as ImapDataResponse, out message)) {
          Trace.Dump(message, 0);
          messages.Add(message);
        }
      }
      else if (response is ImapStatusResponse) {
        var status = response as ImapStatusResponse;

        if (status.Tag != LastTag)
          return;

        if (status.Status == ImapProtocolStatus.Ok) {
          Conclude(new ImapMessageResponse(ImapTransactionStatus.Ok, messages.ToArray(), string.Format("fetched {0} message(s)", messages.Count)));
        }
        else {
          ConcludeError(status);
        }
      }
    }

    private List<ImapMessage> messages = new List<ImapMessage>();
  }
}
