using System;

using Smdn.Protocols.Imap4.Protocol;

namespace Smdn.Protocols.Imap4.Transaction.Transactions {
  internal class StartTlsTransaction : TransactionBase {

    // ProcessIncalable
    //   x Conclude
    // ProcessBadRequest
    //   x Conclude
    // ProcessStartTls
    //   ->WaitForStartTls
    // WaitForStartTls
    //   x Conclude

    public StartTlsTransaction(ImapConnection connection, ImapRequest request)
      : base(connection, request)
    {
    }

    protected override ProcessTransactionDelegate Reset()
    {
      if (!Connection.ServerInfo.Capabilities.Has(ImapKnownCapabilities.StartTls))
        return ProcessIncalable;
      else if (!(Request is ImapStartTlsRequest))
        return ProcessBadRequest;
      else
        return ProcessStartTls;
    }

    private void ProcessBadRequest()
    {
      Conclude(new ImapResponse(ImapTransactionStatus.BadRequest, "bad request"));
    }

    private void ProcessStartTls()
    {
      Send("starttls", WaitForStartTls);
    }

    private void WaitForStartTls()
    {
      ImapProtocolResponse response;

      if (!Receive(out response))
        return;

      if (response is ImapStatusResponse) {
        var status = response as ImapStatusResponse;

        if (status.Tag != LastTag)
          return;

        if (status.Status == ImapProtocolStatus.Ok) {
          // 2.1. STARTTLS Command
          //       A [TLS] negotiation begins immediately after the CRLF at the end
          //       of the tagged OK response from the server.  Once a client issues a
          //       STARTTLS command, it MUST NOT issue further commands until a
          //       server response is seen and the [TLS] negotiation is complete.
          Connection.UpgradeSecurity(Connection.ServerInfo.Uri,
                                     (Request as ImapStartTlsRequest).CreateAuthenticatedStreamCallback);

          //       Once [TLS] has been started, the client MUST discard cached
          //       information about server capabilities and SHOULD re-issue the
          //       CAPABILITY command.
          Connection.ServerInfo.Capabilities = new ImapCapability();

          Conclude(new ImapResponse(ImapTransactionStatus.Ok, "start TLS completed"));
        }
        else {
          ConcludeError(status);
        }
      }
    }
  }
}