using System;

using Smdn.Protocols.Imap4.Protocol;

namespace Smdn.Protocols.Imap4.Transaction.Transactions {
  internal class StatusTransaction : TransactionBase {

    // ProcessArgumentNotSetted
    //   x Conclude
    // ProcessStatus
    //   ->WaitForStatus
    // WaitForStatus
    //   x Conclude

    public StatusTransaction(ImapConnection connection, ImapRequest request)
      : base(connection, request)
    {
    }

    protected override ProcessTransactionDelegate Reset()
    {
      if (!Request.Arguments.ContainsKey("mailbox name"))
        return ProcessArgumentNotSetted;
      else if (!Request.Arguments.ContainsKey("status data item names"))
        return ProcessArgumentNotSetted;
      else
       return ProcessStatus;
    }

    private void ProcessArgumentNotSetted()
    {
      Conclude(new ImapResponse(ImapTransactionStatus.BadRequest, "arguments 'mailbox name' and 'status data item names' must be setted"));
    }

    private void ProcessStatus()
    {
      // TODO: \Noselectはstatusできない?  
      Send("status",
           WaitForStatus,
           new ImapQuotedString(ImapEncoding.ToModifiedUTF7String((string)Request.Arguments["mailbox name"])),
           Request.Arguments["status data item names"]);
    }

    private void WaitForStatus()
    {
      ImapProtocolResponse response;

      if (!Receive(out response))
        return;

      if (response is ImapDataResponse) {
        string name;
        int? messages;
        int? recent;
        int? unseen;
        int? uidNext;
        int? uidValidity;

        if (ImapDataResponseConverter.FromStatusResponse(response as ImapDataResponse,
                                                                 out name,
                                                                 out messages,
                                                                 out recent,
                                                                 out unseen,
                                                                 out uidNext,
                                                                 out uidValidity)) {
          if (mailbox == null)
            mailbox = ImapMailboxManager.GetMailbox(Connection.ServerInfo, (string)Request.Arguments["mailbox name"]);

          if (mailbox.Name.Equals(name)) {
            if (messages.HasValue)      mailbox.ExistsMessage = (int)messages;
            if (recent.HasValue)        mailbox.RecentMessage = (int)recent;
            if (unseen.HasValue)        mailbox.UnseenMessage = (int)unseen;
            if (uidNext.HasValue)       mailbox.UidNext = (int)uidNext;
            if (uidValidity.HasValue)   mailbox.UidValidity = (int)uidValidity;
          }
          else {
            Conclude(new ImapResponse(ImapTransactionStatus.BadRequest, "status comprehension error occurred"));
          }
        }
        else {
          Conclude(new ImapResponse(ImapTransactionStatus.BadRequest, "status comprehension error occurred"));
        }
      }
      else if (response is ImapStatusResponse) {
        var status = response as ImapStatusResponse;

        if (status.Tag != LastTag)
          return;

        Trace.Dump(mailbox);

        if (status.Status == ImapProtocolStatus.Ok) {
          Conclude(new ImapMailboxResponse(ImapTransactionStatus.Ok, mailbox, "status succeeded"));
        }
        else if (status.Status == ImapProtocolStatus.No) {
          // RFC 2193 IMAP4 Mailbox Referrals
          // http://tools.ietf.org/html/rfc2193
          // 4.1. SELECT, EXAMINE, DELETE, SUBSCRIBE, UNSUBSCRIBE, STATUS and APPEND
          //      Referrals
          ImapUri[] referrals;
          if (ImapStatusResponseConverter.FromStatusNoResponse(status, out referrals))
            Conclude(new ImapMailboxReferralResponse(ImapTransactionStatus.MailboxReferral, referrals, status.Text));
          else
            ConcludeError(status);
        }
        else {
          ConcludeError(status);
        }
      }
    }

    private ImapMailbox mailbox = null;
  }
}
