using System;
using System.Collections.Generic;

using Smdn.Protocols.Imap4.Protocol;

namespace Smdn.Protocols.Imap4.Transaction.Transactions {
  internal class StoreTransaction : TransactionBase {

    // ProcessArgumentNotSetted
    //   x Conclude
    // ProcessStore
    //   ->WaitForStore
    // WaitForStore
    //   x Conclude

    public StoreTransaction(ImapConnection connection, ImapRequest request)
      : base(connection, request)
    {
    }

    protected override ProcessTransactionDelegate Reset()
    {
      if (!Request.Arguments.ContainsKey("message data item name"))
        return ProcessArgumentNotSetted;
      else if (!(Request.Arguments.ContainsKey("uid set") || Request.Arguments.ContainsKey("sequence set")))
        return ProcessArgumentNotSetted;
      else
        return ProcessStore;
    }

    private void ProcessArgumentNotSetted()
    {
      Conclude(new ImapResponse(ImapTransactionStatus.BadRequest, "arguments 'message data item name', and 'uid set' or 'sequence set' must be setted"));
    }

    private void ProcessStore()
    {
      if (Request.Arguments.ContainsKey("uid set"))
      // UID STORE
        Send("uid store",
             WaitForStore,
             Request.Arguments["uid set"],
             Request.Arguments["message data item name"]);
      else
        // STORE
        Send("store",
             WaitForStore,
             Request.Arguments["sequence set"],
             Request.Arguments["message data item name"]);
    }

    private void WaitForStore()
    {
      ImapProtocolResponse response;

      if (!Receive(out response))
        return;

      if (response is ImapDataResponse) {
        ImapMessage message;
        if (ImapDataResponseConverter.FromStoreCommandResponse(response as ImapDataResponse, out message)) {
          Trace.Dump(message, 0);
          messages.Add(message);
        }
      }
      else if (response is ImapStatusResponse) {
        var status = response as ImapStatusResponse;

        if (status.Tag != LastTag)
          return;

        if (status.Status == ImapProtocolStatus.Ok) {
          Conclude(new ImapMessageResponse(ImapTransactionStatus.Ok, messages.ToArray(), string.Format("stored {0} message(s)", messages.Count)));
        }
        else {
          ConcludeError(status);
        }
      }
    }

    private List<ImapMessage> messages = new List<ImapMessage>();
  }
}