using System;

using Smdn.Protocols.Imap4.Protocol;

namespace Smdn.Protocols.Imap4.Transaction.Transactions {
  internal class UnselectTransaction : TransactionBase {

    // ProcessUnselect
    //   ->WaitForUnselect
    // ProcessIncalable
    //   x Conclude
    // WaitForUnselect
    //   x Conclude

    public UnselectTransaction(ImapConnection connection, ImapRequest request)
      : base(connection, request)
    {
    }

    protected override ProcessTransactionDelegate Reset()
    {
      if (!Connection.ServerInfo.Capabilities.Has(ImapKnownCapabilities.Unselect))
        return ProcessIncalable;
      else
        return ProcessUnselect;
    }

    private void ProcessUnselect()
    {
      Send("unselect", WaitForUnselect);
    }

    private void WaitForUnselect()
    {
      ImapProtocolResponse response;

      if (!Receive(out response))
        return;

      if (response is ImapStatusResponse) {
        var status = response as ImapStatusResponse;

        if (status.Tag != LastTag)
          return;

        if (status.Status == ImapProtocolStatus.Ok) {
          Conclude(new ImapResponse(ImapTransactionStatus.Ok, "unselect completed"));
        }
        else {
          ConcludeError(status);
        }
      }
    }
  }
}
