using System;
using System.Collections.Generic;

using Smdn.Protocols.Pop3.Protocol;

namespace Smdn.Protocols.Pop3.Transaction.Transactions {
  internal class CapaTransaction : TransactionBase {
    // ProcessCapa
    //   ->WaitForCapa
    // WaitForCapa
    //   x Conclude
    // RetrieveCapa
    //   x Conclude

    public CapaTransaction(PopConnection connection, PopRequest request)
      : base(connection, request)
    {
    }

    protected override ProcessTransactionDelegate Reset()
    {
      return ProcessCapa;
    }

    private void ProcessCapa()
    {
      Send("CAPA");

      Transit(WaitForCapa);
    }

    private void WaitForCapa()
    {
      PopProtocolResponse response;

      if (!Receive(out response))
        return;

      if (response is PopStatusResponse) {
        var status = response as PopStatusResponse;

        if (status.Status == PopProtocolStatus.Positive) {
          knownCapabilities |= PopKnownCapabilities.Capabilities;
          Transit(RetrieveCapa);
        }
        else {
          Connection.ServerInfo.Capabilities.KnownCapabilities &= ~PopKnownCapabilities.Capabilities;
          ConcludeError(status);
        }
      }
    }

    private void RetrieveCapa()
    {
      PopProtocolResponse response;

      if (!Receive(out response))
        return;

      if (response is PopFollowingResponse) {
        RetrieveCapability(response as PopFollowingResponse);
      }
      else if (response is PopTerminationResponse) {
        var capability = new PopCapability(knownCapabilities, allCapabilities.ToArray());
        capability.LoginDelay = loginDelay;
        capability.Expire = expire;

        Connection.ServerInfo.Capabilities.KnownCapabilities |= capability.KnownCapabilities;
        Connection.ServerInfo.Capabilities.AllCapabilities = capability.AllCapabilities;
        Connection.ServerInfo.Capabilities.LoginDelay = capability.LoginDelay;
        Connection.ServerInfo.Capabilities.Expire = capability.Expire;

        Conclude(new PopCapabilityResponse(PopTransactionStatus.Ok,
                                           capability,
                                           "comprehended server capabilities"));
      }
    }

    private void RetrieveCapability(PopFollowingResponse following)
    {
      allCapabilities.Add(following.Text);

      if (following.Text.Contains(" ")) {
        var args = following.Text.Split(new[] {' '}, StringSplitOptions.RemoveEmptyEntries);

        switch (args[0]) {
          case "SASL":
          {
            foreach (var cap in args) {
              switch (cap) {
                case "LOGIN":
                  knownCapabilities |= PopKnownCapabilities.AuthLogin;
                  break;
                case "PLAIN":
                  knownCapabilities |= PopKnownCapabilities.AuthPlain;
                  break;
                case "CRAM-MD5":
                  knownCapabilities |= PopKnownCapabilities.AuthCramMd5;
                  break;
                case "KERBEROS_V4":
                  knownCapabilities |= PopKnownCapabilities.AuthKerberosV4;
                  break;
              }
            }
            return;
          }

          case "LOGIN-DELAY":
            knownCapabilities |= PopKnownCapabilities.LoginDelay;
            loginDelay = args[1];
            return;

          case "EXPIRE":
          {
            knownCapabilities |= PopKnownCapabilities.Expire;
            expire = args[1];
            return;
          }
        }
      }

      switch (following.Text) {
        case "CAPA":
          // knownCapabilities |= PopKnownCapabilities.Capabilities;
          break;
        case "TOP":
          knownCapabilities |= PopKnownCapabilities.Top;
          break;
        case "USER":
          knownCapabilities |= PopKnownCapabilities.User;
          break;
        case "RESP-CODES":
          knownCapabilities |= PopKnownCapabilities.RespCodes;
          break;
        case "PIPELINING":
          knownCapabilities |= PopKnownCapabilities.Pipelining;
          break;
        case "UIDL":
          knownCapabilities |= PopKnownCapabilities.Uidl;
          break;
        case "IMPLEMENTATION":
          knownCapabilities |= PopKnownCapabilities.Implementation;
          break;
        case "STLS":
          knownCapabilities |= PopKnownCapabilities.Stls;
          break;
      }
    }

    private PopKnownCapabilities knownCapabilities = PopKnownCapabilities.None;
    private readonly List<string> allCapabilities = new List<string>();
    private string loginDelay = null;
    private string expire = null;
  }
}
