using System;

using Smdn.Protocols.Pop3.Protocol;

namespace Smdn.Protocols.Pop3.Transaction.Transactions {
  internal class UserTransaction : TransactionBase {

    // ProcessUserIncapable
    //   x Conclude
    // ProcessParameterNotSetted
    //   x Conclude
    // ProcessUser
    //   ->WaitForUser
    // WaitForUser
    //   x Conclude

    public UserTransaction(PopConnection connection, PopRequest request)
      : base(connection, request)
    {
    }

    protected override ProcessTransactionDelegate Reset()
    {
      if (Connection.ServerInfo.Capabilities.Has(PopKnownCapabilities.Capabilities) &&
          !Connection.ServerInfo.Capabilities.Has(PopKnownCapabilities.User))
        return ProcessUserIncapable;
      else if (Request.Parameters.ContainsKey("Mailbox"))
        return ProcessUser;
      else
        return ProcessParameterNotSetted;
    }

    private void ProcessUserIncapable()
    {
      Conclude(new PopResponse(PopTransactionStatus.Incapable, "USER incapable"));
    }

    private void ProcessParameterNotSetted()
    {
      Conclude(new PopResponse(PopTransactionStatus.BadRequest, "parameter 'Mailbox' must be setted"));
    }

    private void ProcessUser()
    {
      Send("USER", Request.Parameters["Mailbox"]);

      Transit(WaitForUser);
    }

    private void WaitForUser()
    {
      PopProtocolResponse response;

      if (!Receive(out response))
        return;

      if (response is PopStatusResponse) {
        var status = response as PopStatusResponse;

        if (status.Status == PopProtocolStatus.Positive) {
          Conclude(new PopResponse(PopTransactionStatus.Ok, "succeeded"));
        }
        else {
          ConcludeError(status);
        }
      }
    }
  }
}
